local socket = require("socket")
local url = require("socket.url")

-- to talk to love task
local id = 0

function newTask(t)
	id = id + 1
	body = {}		
	for k,v in pairs(t) do
		table.insert(body, k .. '=' .. url.escape(v))
	end
	body = table.concat(body, '&')
	other:set(id, body)
	print("new task", id, body)
	return id
end

function taskFinished(id)
	return main:peek(id) ~= nil
end

function taskResult(id)
	return JSON:decode(main:get(id))
end

function waitForTask(id)
	while true do
		if taskFinished(id) then
			return taskResult(id)
		end
	end
end

-- end of api

Connection = class("Connection")

function Connection.static:createPlayer(t)
	t.type = "createPlayer"
	local v = waitForTask(newTask(t))
	if type(v) ~= "string"  then
		os.exit(1)
	end
	return v
end

function Connection.static:getPlayer(t)
	t.type = "getPlayer"
	local v = waitForTask(newTask(t))
	if type(v) ~= "table" then
		os.exit(1)
	end
	return v
end

function Connection.static:setPlayer(t)
	t.type = "setPlayer"
	return waitForTask(newTask(t))
end

function Connection.static:login(t)
	t.type = "login"
	return waitForTask(newTask(t))
end

function Connection.static:createMatchup(t)
	t.type = "createMatchup"
	local v = waitForTask(newTask(t))
	if type(v) ~= "string"  then
		os.exit(1)
	end
	return v
end

function Connection.static:joinMatchup(t)
	t.type = "joinMatchup"
	local v = waitForTask(newTask(t))
	if type(v) ~= "table" then
		os.exit(1)
	end
	return v
end

function Connection.static:listMatchup()
	local v = waitForTask(newTask({type = "listMatchup"}))
	if type(v) ~= "table" then
		os.exit(1)
	end
	return v
end

function Connection.static:listPlayer()
	local v = waitForTask(newTask({type = "listPlayer"}))
	if type(v) ~= "table" then
		os.exit(1)
	end
	return v
end

function Connection.static:push(t)
	--add task
	t.type = "push"
	return newTask(t)
end

function Connection.static:pull(t)
	--add task	
	t.type = "pull"
	return newTask(t)
end

function Connection.static:leave(t)
	t.type = "leave"
	return waitForTask(newTask(t))
end

function Connection.static:version()
	local v = waitForTask(newTask({type = "version"}))
	if type(v) ~= "string"  then
		os.exit(1)
	end
	return v
end
