Lobby = class("Lobby")
Lobby.static.Border = 20
Lobby.static.Col = 100
Lobby.static.Button = 150
Lobby.static.Thresh = 10
Lobby.static.Client = "v0.3"
Lobby.static.AvatarCount = 16

function Lobby:initialize()
	-- in reverse order
	self.matchupPanel = self:createMatchupPanel()
	self.login = self:createLogin()
	self.lobby = self:createGameLobby()
	self.register = self:createRegister()
	self.matchup = nil
	self.thresh = 0
	self.pid = nil
end

function Lobby:draw()
	if not self.matchup then return end
	self.matchup:draw()
end

function Lobby:update(dt)
	if self.matchup then
		self.matchup:update(dt)
		self:updateControlledUnitInfo()
	else
		self.thresh = self.thresh - dt
		if self.thresh < 0 then
			self.thresh = Lobby.Thresh

			self:updatePlayerList()
			self:updateMatchupList()
		end
	end
end

function Lobby:getMouse(x, y, button)
	if not self.matchup then return end
	local match = self.matchup
	
	if button == "r" then
		match:moveControlledUnitByMouse(x, y)
	end
end

function Lobby:getKey(key, unicode)
	
	if not self.matchup then return end
	
	local match = self.matchup
	if key == "up" or key == "down" or key == "left" or key == "right" then
		match:moveControlledUnit(key)
	end
	
	if key == "tab" then
		match:nextControllUnit()
	end
	
	if key == "z" then
		match:controlledUnitAttack()
	end
	
	if key == "x" then
		match:controlledUnitSpecialAttack()
	end
	
	if key == "c" then
		match:moveControlledUnitUp()
	end
end

function Lobby:prepareMatchup()
	if not self.pid then return end
	self.matchup:insertUnit(self.pid)
end

function Lobby:createRegister()
	local register = loveframes.Create("frame")
	register:SetVisible(false)
	register:SetSize(6 * Lobby.Col, 5 * Lobby.Col)
	register:Center()
	register:SetName("Get Started! Register")
	
	
	local image = loveframes.Create("image", register)
	image:SetImage("resource/cg/character.png")
	image:SetPos(Lobby.Border, 2 * Lobby.Border)
	
	local panel = loveframes.Create("panel", register)
	panel:SetSize(register:GetWidth() - image:GetWidth() - 3 * Lobby.Border, register:GetHeight() - 3 * Lobby.Border)
	panel:SetPos(image:GetWidth() + 2 * Lobby.Border, 2 * Lobby.Border)
	
	---- stuff
	
	local form = {
		{
			text = "Account",
		},
		{
			text = "Password",
		},
		{
			text = "Name",
		}
	}
	
	local result = {}
	local lastEleY = 0
	for i, element in ipairs(form) do
		local text = loveframes.Create("text", panel)
		text:SetWidth(panel:GetWidth() - 2 * Lobby.Border)
		text:SetText(element.text)
		
		local textinput = loveframes.Create("textinput", panel)
		textinput:SetWidth(panel:GetWidth() - 2 * Lobby.Border)
		
		text:SetPos(Lobby.Border, Lobby.Border + (i - 1) * (Lobby.Border + text:GetHeight() + textinput:GetHeight()) )
		textinput:SetPos(Lobby.Border, Lobby.Border + (i - 1) * (Lobby.Border + text:GetHeight() + textinput:GetHeight()) + 0.5 * Lobby.Border + text:GetHeight())
		lastEleY = textinput:GetStaticY() + textinput:GetHeight()
		
		
		textinput.OnFocusLost = function(object)
			result[element.text:lower()] = object:GetText()
		end
	end
	
	local logo = loveframes.Create("image", panel)
	logo:SetImage("resource/cg/logo.png")
	logo:SetPos(panel:GetWidth() - Lobby.Border - logo:GetWidth(), panel:GetHeight() - Lobby.Border - logo:GetHeight())
	
	local button = loveframes.Create("imagebutton", panel)
	button:SetSize(Lobby.Col, 40)
	button:SetImage("resource/cg/register.png")
	button:SetPos(panel:GetWidth() - Lobby.Border - button:GetWidth(), lastEleY + Lobby.Border)
	button:SetText("")
	button.OnClick = function(object)
		-- empty
		if table.count(result) ~= #form then return end
		for i, v in pairs(result) do
			--print(i, v)
			if v == "" then
				return
			end
		end
		register:SetVisible(false)
		Connection:createPlayer(result)
		self:updatePlayerList()
	end
	
	local text = loveframes.Create("text", panel)
	text:SetWidth(panel:GetWidth() - 2 * Lobby.Border)
	text:SetPos(Lobby.Border, button:GetStaticY() + button:GetHeight() + Lobby.Border)
	text:SetText("By clicking [register], you have agreed our privacy term.")
	
	return register
end

function Lobby:createGameLobby()
	local base = loveframes.Create("panel")

	base:SetSize(love.graphics.getWidth(), love.graphics.getHeight())
	
	local reserve = 80

	-- player id
	local playerList = loveframes.Create("list", base)
	playerList:SetPos(Lobby.Border, Lobby.Border)
	playerList:SetSize(2 * Lobby.Col, love.graphics.getHeight() - Lobby.Border - reserve)
	playerList:SetDisplayType("vertical")
	playerList:SetPadding(5)
	playerList:SetSpacing(10)
	
	-- matchuplist
	local matchuplist = loveframes.Create("list", base)
	matchuplist:SetPos(3 * Lobby.Border + playerList:GetWidth(), Lobby.Border)
	matchuplist:SetSize(4 * Lobby.Col, love.graphics.getHeight() - Lobby.Border - reserve)
	matchuplist:SetDisplayType("vertical")
	matchuplist:SetPadding(5)
	matchuplist:SetSpacing(10)
	
	local banner = loveframes.Create("image", base)
	banner:SetImage("resource/cg/ads.png")
	banner:SetPos(love.graphics.getWidth() - banner:GetWidth() - 2 * Lobby.Border, Lobby.Border)
	
	local logo = loveframes.Create("image", base)
	logo:SetImage("resource/cg/logo.png")
	logo:SetPos(Lobby.Border, love.graphics.getHeight() - logo:GetHeight() - Lobby.Border)
	
	local version = loveframes.Create("text", base)
	version:SetHeight(40)
	version:SetPos(2 * Lobby.Border + logo:GetWidth(), love.graphics.getHeight() - version:GetHeight() - Lobby.Border)
	version:SetText("client: " .. Lobby.Client .. " server: " .. Connection:version())

	local quit = loveframes.Create("button", base)
	quit:SetSize(60, 40)
	quit:SetPos(love.graphics.getWidth() - quit:GetWidth() - 2 * Lobby.Border, love.graphics.getHeight() - reserve - quit:GetHeight())
	quit:SetText("quit")
	quit.OnClick = function(object)
		love.event.push("quit")
	end
	
	local register = loveframes.Create("button", base)
	register:SetSize(60, 40)
	register:SetPos(love.graphics.getWidth() - register:GetWidth() - 2 * Lobby.Border, love.graphics.getHeight() - reserve - quit:GetHeight() - register:GetHeight())
	register:SetText("register")
	register.OnClick = function(object)
		self.register:SetVisible(true)
		self.register:MoveToTop()
	end
	

	
	local play = loveframes.Create("imagebutton", base)
	play:SetImage("resource/cg/play.png")
	play:SetSize(banner:GetWidth() - Lobby.Border - register:GetWidth(), 80)
	play:SetPos(love.graphics.getWidth() - banner:GetWidth() - 2 * Lobby.Border, love.graphics.getHeight() - reserve - play:GetHeight())
	play:SetText("")
	play.OnClick = function(object)
		if not self.pid then
			self.login:SetVisible(true)
			self.login:MoveToTop()
			return
		end
		base:SetVisible(false)
		
		local tempMap = Map:new()
		tempMap:generate()
		local state = tempMap:export()
		local mid = Connection:createMatchup({pid = self.pid, map = JSON:encode(state) })
		self.matchup = Matchup:new(mid)
		self.matchup.map:import(state)
		self:prepareMatchup()
		self:updateMatchupList()
	end
	
	self.status = loveframes.Create("text", base)
	self.status:SetSize(2 * Lobby.Col, 40)
	self.status:SetPos(love.graphics.getWidth() - self.status:GetWidth() - 2 * Lobby.Border, love.graphics.getHeight() - self.status:GetHeight() - Lobby.Border)
	self.status:SetText('not login')
	
	return base
end


function Lobby:updateLoginStatus()
	if not self.pid then return end
	local info = Connection:getPlayer({pid = self.pid})
	self.status:SetText({"You are logged in as: ", {76, 153 , 0}, info.name})
end

function Lobby:createLogin()
	local base = loveframes.Create("frame")
	base:SetVisible(false)
	base:SetSize(3 * Lobby.Col, 3 * Lobby.Col)
	base:Center()
	base:SetName("Login First.")
	
	
	local form = {
		{
			text = "Account",
		},
		{
			text = "Password",
		},
	}
	
	local result = {}
	local lastEleY = 0
	for i, element in ipairs(form) do
		local text = loveframes.Create("text", base)
		text:SetWidth(base:GetWidth() - 2 * Lobby.Border)
		text:SetText(element.text)
		
		local textinput = loveframes.Create("textinput", base)
		textinput:SetWidth(base:GetWidth() - 2 * Lobby.Border)
		
		text:SetPos(Lobby.Border, 2 * Lobby.Border + (i - 1) * (Lobby.Border + text:GetHeight() + textinput:GetHeight()) )
		textinput:SetPos(Lobby.Border, 2 * Lobby.Border + (i - 1) * (Lobby.Border + text:GetHeight() + textinput:GetHeight()) + 0.5 * Lobby.Border + text:GetHeight())
		lastEleY = textinput:GetStaticY() + textinput:GetHeight()
		
		
		textinput.OnFocusLost = function(object)
			result[element.text:lower()] = object:GetText()
		end
	end
	
	local button = loveframes.Create("imagebutton", base)
	button:SetSize(Lobby.Col, 40)
	button:SetImage("resource/cg/start.png")
	button:SetPos(base:GetWidth() - Lobby.Border - button:GetWidth(), lastEleY + Lobby.Border)
	button:SetText("")
	button.OnClick = function(object)
		-- empty
		if table.count(result) ~= #form then return end
		for i, v in pairs(result) do
			if v == "" then
				return
			end
		end
		local pid = Connection:login(result)
		if pid == "bad" then
			return
		end
		base:SetVisible(false)
		self.pid = pid
		self:updateLoginStatus()
	end
	
	local image = loveframes.Create("image", base)
	image:SetImage("resource/cg/login.png")
	image:SetPos(0, base:GetHeight() - image:GetHeight())
	
	return base
end

function Lobby:createMatchupPanel()
	local infoPanel = loveframes.Create("panel")
	infoPanel:SetSize(3 * Lobby.Col , 5 * Lobby.Border + 30 + 40)
	infoPanel:SetPos(Lobby.Border, love.graphics.getHeight() - infoPanel:GetHeight() - Lobby.Border)

	
	local action = {
		{
			image = "resource/cg/lobby.png",
			handler = function(object)
				self.lobby:SetVisible(true)
				self.lobby:MoveToTop()
				self.matchup = nil
				Connection:leave({pid = self.pid})
				self:updateMatchupList()
			end
		},
		{
			image = "resource/cg/attack.png",
			handler = function(object)
				self.matchup:controlledUnitAttack()
			end
		},
		{
			image = "resource/cg/spattack.png",
			handler = function(object)
				self.matchup:controlledUnitSpecialAttack()
			end
		},
		{
			image = "resource/cg/dodge.png",
			handler = function(object)
				self.matchup:moveControlledUnitUp()
			end
		},
	}
	
	for i, element in ipairs(action) do
		local button = loveframes.Create("imagebutton")
		button:SetSize(50, 50)
		button:SetImage(element.image)
		button:SetPos(love.graphics.getWidth() - i * (button:GetWidth() + Lobby.Border), love.graphics.getHeight() - Lobby.Border - button:GetHeight())
		button:SetText("")
		button.OnClick = element.handler
	end
	return infoPanel
end


function Lobby:updatePlayerList()
	local playerList = self.lobby.children[1]
	playerList.children = {}
	
	local getIconId = function(name)
		local sum = 0
		for i = 1, #name do
			sum = 128 * sum + string.byte(name, i)
		end
		return (sum % Lobby.AvatarCount) + 1
	end
	for i, v in pairs(Connection:listPlayer()) do
		local panel = loveframes.Create("panel", playerList)
		panel:SetHeight(40)
		
		local image = loveframes.Create("image", panel)
		image:SetImage("resource/avatar/" .. getIconId(v) .. ".png")
		image:SetPos(0, 0)
		
		local text = loveframes.Create("text", panel)
		text:SetPos(image:GetWidth() + Lobby.Border, 0.5 * Lobby.Border)
		text:SetText(v)
	end
end

function Lobby:updateMatchupList()
	local matchupList = self.lobby.children[2]
	matchupList.children = {}
	for i, v in pairs(Connection:listMatchup()) do
		local panel = loveframes.Create("panel", matchupList)
		panel:SetHeight(40)
		local text = loveframes.Create("text", panel)
		text:SetPos(Lobby.Border, 0.5 * Lobby.Border)
		text:SetText(v)
		
		local button = loveframes.Create("imagebutton", panel)
		button:SetImage("resource/cg/enter.png")
		button:SetSize(Lobby.Col, panel:GetHeight())
		button:SetPos(matchupList:GetWidth() - button:GetWidth() - 10 , 0)
		button:SetText("")
		button.OnClick = function(object)
			if not self.pid then
				self.login:SetVisible(true)
				self.login:MoveToTop()
				return
			end
			self.lobby:SetVisible(false)
			local state = Connection:joinMatchup({pid = self.pid, mid = i})
			self.matchup = Matchup:new(i)
			self.matchup.map:import(state)
			self:prepareMatchup()
		end
		
	end
end

function Lobby:updateControlledUnitInfo()
	if not self.matchup then return end
	
	local u = self.matchup:getControlledUnit()
	local unit = {
		{
			text = "HP",
			value = u.HP,
			max = 100,
		},
		{
			text = "ATK",
			value = u.attack,
			max = 12,
		},
		{
			text = "DEF",
			value = u.defense,
			max = 12,
		},
		{
			text = "RANGE",
			value = u.range,
			max = 10,
		},
		{
			text = "ENERGY",
			value = u.energy,
			max = 100,
		},
		--{"weapon", u.weapon.name},
		--{"note", u.weapon.info},
	}
	
	self.matchupPanel.children = {}
	local image = loveframes.Create("image", self.matchupPanel)
	image:SetImage(WorldObj.Skin[5])
	image:SetPos(5,  5)
	
	local image = loveframes.Create("image", self.matchupPanel)
	image:SetImage(WorldObj.Skin[u.weapon.skin])
	image:SetPos(5,  5)
	
	
	local weapon = loveframes.Create("text", self.matchupPanel)
	weapon:SetSize(self.matchupPanel:GetWidth() - 15 - image:GetWidth(), image:GetHeight())
	weapon:SetPos(10 + image:GetWidth(),  5)
	weapon:SetText("[" .. u.weapon.name .. "] " ..u.weapon.info)
	
	for i, v in ipairs(unit) do
		local panel = loveframes.Create("panel", self.matchupPanel)
		panel:SetSize(3 * Lobby.Border, Lobby.Border)
		panel:SetPos(5, (Lobby.Border + 5) * (i - 1) + image:GetHeight() + 5 +  5)
		local text = loveframes.Create("text", panel)
		text:SetPos(5,  5)
		text:SetText(v.text)
		
		local bar = loveframes.Create("progressbar", self.matchupPanel)
		bar:SetSize(self.matchupPanel:GetWidth() - 15 - panel:GetWidth(), Lobby.Border)
		bar:SetPos(panel:GetWidth() + 10, (Lobby.Border + 5) * (i - 1) + image:GetHeight() + 5 +  5)
		bar:SetMax(v.max)
		bar:SetValue(v.value)
	end
end
