Matchup = class("Matchup")
Matchup.static.Sync = 2

function Matchup:initialize(mid)
	self.mid = mid
    self.map = Map:new()    
    self.unit = {}
    self.otherUnit = {}
    self.controlledUnit = nil
    
	self.sync = 0
	self.pullQueue = {}
	self.pullDone = {}
end

function Matchup:insertUnit(id)
	local u = Unit:new(id)
	table.insert(self.unit, u)
	self.controlledUnit = #self.unit
	self.map:insert(u)
end

function Matchup:insertOtherUnit(id)
	local u = Unit:new(id)
	table.insert(self.otherUnit, u)
	self.map:insert(u)
end

function Matchup:getControlledUnit()
	return self.unit[self.controlledUnit]
end

function Matchup:moveControlledUnitUp()
	local u = self:getControlledUnit()
	if u.HP <= 0 then return end
	self.map:moveUp(u)
end

function Matchup:moveControlledUnitByMouse(px, py)
	local obj = self:getControlledUnit()
	if obj.HP <= 0 then return end
	local x, y = self.map:toCoord(px, py, obj.x, obj.y)
	if x == -1 and y == -1 then
		return 
	end
	local diffx = math.abs(obj.x - x)
	local diffy = math.abs(obj.y - y)
	if diffx > diffy then
		--move x
		if obj.x > x then
			self.map:moveLeft(obj)
		end
		
		if obj.x < x then
			self.map:moveRight(obj)
		end
	else
		--move y
		if obj.y > y then
			self.map:moveForward(obj)
		end
		
		if obj.y < y then
			self.map:moveBackward(obj)
		end
	end
	self:push({
		pid = obj.id,
		action = 1
	})
end

function Matchup:moveControlledUnit(key)
	local u = self:getControlledUnit()
	if u.HP <= 0 then return end
	if key == "left" then
		self.map:moveLeft(u)
	elseif key == "right" then
		self.map:moveRight(u)
	elseif key == "down" then
		self.map:moveBackward(u)
	elseif key == "up" then
		self.map:moveForward(u)
	end
	self:push({
		pid = u.id,
		action = 1
	})
end

function Matchup:findUnitById(id)
	for i, v in ipairs(self.otherUnit) do
		if v.id == id then
			return v
		end
	end
	return nil
end

function Matchup:push(body)
	--print ("PUSH")
	body.mid = self.mid
	Connection:push(body)
end


function Matchup:doPull()
	for i, v in ipairs(self.pullQueue) do
		if taskFinished(v) then
			table.remove(self.pullQueue, i)
			local value = taskResult(v)
			print("insert to done:", value)
			table.insert(self.pullDone, value)
		end
	end
end

function Matchup:pull()
	--print ("PULL")
	table.insert(self.pullQueue, Connection:pull({mid = self.mid}))
end

function Matchup:update(dt)	
	local u = self:getControlledUnit()
	
    self.map:update(dt)
    self.map.viewer = u
    
	self:doPull()
  
    self.sync = self.sync -dt
    if self.sync < 0 then
		self.sync = Matchup.Sync	
		
		--push basic info
		self:push({
			pid = u.id,
			x = u.x,
			y = u.y,
			HP = u.HP
		})
		
		self:pull()
	end
	
	--pull
	if #self.pullDone > 0 then
		-- get the lastest, state before this is too laggy
		local state = table.remove(self.pullDone, #self.pullDone)
		self.pullDone = {}
		print_r(state)
		for i, v in ipairs(state) do
			if v and v.id and v.x and v.y and v.HP then
				if v.id == u.id then
					-- dont update location but update health
					u.HP = v.HP
				else
					-- enemy
					local a = self:findUnitById(v.id)
					if a then
						self.map:move(a, v.x, v.y)
						a.HP = v.HP
						--print(v.action, a.id)
						if v.id ~= u.id and v.action then
							if v.action == 1 then
							elseif v.action == 2 then
								print("attack " .. v.id)
								a:attackAction(self)
							elseif v.action == 3 then
								print("special attack " .. v.id)
								a:specialAttackAction(self)
							end
						end
					else
						-- new guy comes
						print ("insert unit: " .. v.id )
						self:insertOtherUnit(v.id)
					end
				end
			end
		end
		
		-- end of pull done
	end
end

function Matchup:controlledUnitAttack()
	local u = self:getControlledUnit()
	if u.HP <= 0 then return end
	u:attackAction(self)
end

function Matchup:controlledUnitSpecialAttack()
	-- add special effect
	local u = self:getControlledUnit()
	if u.HP <= 0 then return end
	u:specialAttackAction(self)
end

function Matchup:draw()
	local u = self:getControlledUnit()
	if u.HP <= 0 then
		love.graphics.setColor(255, 0, 0)
		love.graphics.print("You are dead. You may leave the room.", Lobby.Border, Lobby.Border)
		love.graphics.reset()
	end

	self.map:draw(u.x, u.y)
	for i, v in ipairs(self.otherUnit) do
		v:say(u.x, u.y)
		v:sayName(u.x, u.y)
	end
	for i, v in ipairs(self.unit) do
		v:say(u.x, u.y)
		v:sayName(u.x, u.y)
	end
end

function Matchup:nextControllUnit()
	key = self.controlledUnit + 1
	if key > #self.unit then
		key = 1
	end
	self.controlledUnit = key
end
