Unit = class("Unit", WorldObj)
Unit.static.Memory = 4

function Unit:initialize(pid)
	local t = Connection:getPlayer({pid = pid})
	print_r(t)
	WorldObj.initialize(self, 5, 1, 1, -1)
	self.HP = math.round(100 * math.log10(t.HP))
	self.attack = t.attack
	self.defense = t.defense
	self.range = t.range
	self.energy = math.round(100 * math.log10(t.energy))
	self.sight = 5
	self.id = t._id
	self.name = t.name
	
	self.Memory = 0
	self.says = {}
	self.weapon = Weapon:new(t.weaponId)
	self.weapon.modifier(self)
end

function Unit:draw(offx, offy)
	WorldObj.draw(self, offx, offy)
	self:drawWeapon(offx, offy)
end

function Unit:sayName(offx, offy)
	love.graphics.setColor(255, 255, 255)
	local px, py = self:toPixel(offx, offy)
	love.graphics.print(self.name, px , py + Map.BlockSize);
	love.graphics.reset()
end

function Unit:say(offx, offy)
	if #self.says == 0 then return end
	love.graphics.setColor(255, 0, 0)
	local px, py = self:toPixel(offx, offy)
	for i = #self.says, #self.says - 5, -1 do
		if i == 0 then break end
		love.graphics.print(self.says[i], px + 5 * i, py - i * 15);
	end
	love.graphics.reset()
end

function Unit:drawWeapon(offx, offy)
	local px, py = self:toPixel(offx, offy)
	love.graphics.draw(WorldObj.Skin[self.weapon.skin], px, py)
end

function Unit:update(dt)
    self.Memory = self.Memory - dt
    if self.Memory < 0 then
		self.says = {}
		self.Memory = Unit.Memory
	end
end

function Unit:damage(o, match)
	local damage = math.ceil(self.attack / o.defense * 5)
	o.HP = o.HP - damage
	match.map:insert(AniObj:new(0.5, self.weapon.effect, o.x, o.y, o.z))
	print("damage")
	
	match:push({
		pid = o.id,
		HP = o.HP,
	})


	if o.HP <= 0 then
		o.HP = 0
		table.insert(self.says, "SLAYER!!")
		table.insert(o.says, "KILLED :(")
		o.name = "[DEAD]"
	else
		table.insert(self.says, (#self.says + 1) .. " hits")
		table.insert(o.says, "HP " .. o.HP)
	end
end


function Unit:attackAction(match)
	-- pick one group to be in
	local others = match.otherUnit
	for i, v in ipairs(others) do
		if v == self then
			-- enemy is actually = unit
			others = match.unit
			break
		end
	end

	local u = self
	for i, v in ipairs(others) do
		local distance = u:distanceFrom(v)
		if u ~= v and distance <= u.range and distance <= u.sight then
			match.map:moveUp(v)
			self:damage(v, match)
		end
	end
	match.map:moveUp(u)
	
	match:push({
		pid = u.id,
		action = 2
	})
end

function Unit:specialAttackAction(match)
	local u = self
	self:attackAction(match)
	for i, v in ipairs(match.map:near(u, u.range)) do
		if u:distanceFrom(v) < u.range then 
			match.map:moveUp(v)
		end
	end
	
	match:push({
		pid = u.id,
		action = 3
	})
end

