WorldObj = class("WorldObj")

WorldObj.static.Skin = {
	love.graphics.newImage("resource/image/grasstile.png"),
	love.graphics.newImage("resource/image/treetile.png"),
	love.graphics.newImage("resource/image/stonetile.png"),
	love.graphics.newImage("resource/image/watertile.png"),
	love.graphics.newImage("resource/image/unit.png"),
	love.graphics.newImage("resource/image/arrow.png"),
	love.graphics.newImage("resource/image/area.png"),
	love.graphics.newImage("resource/image/area2.png"),
	
	love.graphics.newImage("resource/image/sword.png"),
	love.graphics.newImage("resource/image/slashsprite.png"),
	
	love.graphics.newImage("resource/image/fork.png"),
	love.graphics.newImage("resource/image/firesprite.png"),
}

function WorldObj:initialize(skin, x, y, z)
	self.skin = skin
	self.x = x
	self.y = y
	self.z = z
	self.weight = 1
	self.layer = 1
	self.dead = false
end

function WorldObj:distanceFrom(worldObj)
	return math.sqrt((self.x - worldObj.x) ^ 2 + (self.y- worldObj.y) ^ 2 + (self.z- worldObj.z) ^ 2)
end

function WorldObj.static:distance(x1, y1, x2, y2)
	return math.sqrt((x1 - x2)^2 + (y1 - y2)^2)
end

function inFrontOf(obj1, obj2)
	if obj1.x == obj2.x and obj1.y == obj2.y and obj1.z == obj2.z and obj1.layer < obj2.layer then
        return true
    elseif obj1.z < obj2.z then
		return true
	elseif obj1.x + obj1.y < obj2.x + obj2.y and obj1.z == obj2.z then
		return true
	else
		return false
	end
end

function WorldObj:toPixel(offx, offy)
	local block_width = Map.BlockSize
    local block_depth = Map.BlockSize / 2
	local grid_x = (love.graphics.getWidth() / 2) - (block_width / 2) * (offx - offy)
	local grid_y = (love.graphics.getHeight() / 3) - (block_depth / 2) * (offx + offy)
	
	local px = math.round(grid_x + (self.x-self.y) * (block_width / 2))
	local py = math.round(grid_y + (self.x+self.y) * (block_depth / 2) - (self.z - 1) * block_depth)
	return px, py
end

function WorldObj:draw(offx, offy)
	local px, py = self:toPixel(offx, offy)
	love.graphics.draw(WorldObj.Skin[self.skin], px, py)
end

function WorldObj:clicked(px, py, offx, offy)
	local block_width = Map.BlockSize
    local block_depth = Map.BlockSize / 2
	local apx, apy = self:toPixel(offx, offy)
	-- adjust by 1/2 grid unit
	-- y : click on ground top so another 1/2
	return WorldObj:distance(px - block_width / 4, py - block_depth / 8, apx, apy) < block_depth / 2
end

function WorldObj:cover(worldObj)
	local diff = self.x - worldObj.x
	return diff > 0 and self.y == worldObj.y + diff and self.z == worldObj.z + diff
end

function WorldObj:update(dt)
	-- nothing
end
